#ifndef __EFI_h

#define	__EFI_h





/* Define byte, word, and long word types for sci buffer routines

*/

#ifndef __POSIX_SYS_TYPES_h

typedef unsigned char u_char; 

typedef unsigned short int u_short;

typedef unsigned long u_long; 

#endif





/* fixed point stuff

*/

typedef long fixed;			/* Our new fixed point type.	*/

#define itofx(x) ((x) << 8)		/* Integer to fixed point	*/

#define ftofx(x) ((x) * 256)		/* Float to fixed point		*/

#define dtofx(x) ((x) * 256)		/* Double to fixed point	*/

#define fxtoi(x) ((x) >> 8)		/* Fixed point to integer	*/

#define fxtof(x) ((float) (x) / 256)	/* Fixed point to float		*/

#define fxtod(x) ((double)(x) / 256)	/* Fixed point to double	*/

#define Mulfx(x,y) (((y) * (x)) >> 8)	/* Multiply a fixed by a fixed	*/

#define Divfx(x,y) ((y << 8) / (x))    	/* Divide a fixed by a fixed	*/





/* clock speeds, interrupt vector offsets and levels, arbitration values

*/

#define	FCRYSTAL	32768L

#define	SYSCLK_W	0

#define	SYSCLK_X	1

#define	SYSCLK_Y	63

#define SYSCLK(W,X,Y) ((FCRYSTAL * 4 * (Y + 1) * (1 << (2 * W + X) )))



#define SCI_BAUD            19200	/*  19.2kbps  */     /* 57600 57.6 kbps  */

#define QSPI_BAUD           209000    /* 209 kHz    */

#define PIT_RATE            5         /* periodic interrupt timer rate  */

#define PIT_PRESCALE        1         /* use prescale = 1 for freqs > 3.98 hz */





static unsigned int system_clock = SYSCLK(1,1,23);

#define scidivider(baud) (((system_clock >> 5)+((long)(baud)/2))/(baud))

#define qspidivider(baud) (((system_clock >> 1)+((long)(baud)/2))/(baud))

#define pitdivider(freq) (((FCRYSTAL >> 2)+((long)((PIT_PRESCALE << 9)*freq)/2))/((PIT_PRESCALE << 9)*freq))







#define SIMBASE             0xFFFA00  

#define RAMBASE             0xFFFB00  

#define QSMBASE             0xFFFC00  

#define QSMDATABASE         0xFFFD00  

#define TPUREGBASE          0xFFFE00  

#define TPUDPBASE           0xFFFF00  

#define EMURAMBASE          0xFFF000  





#define VECTORBASE          0x00000     /* Vector Base Register value */

#define SCIVEC              0x56		/* SCI receive vector number  */

#define QSPIVEC             0x57        

#define TPUVEC              0x58    

#define PITVEC              0x40



#define SCI_IRQL            0x6

#define PIT_IRQL            0x7

#define QSPI_IRQL           0x5

#define TPU_IRQL            0x4



#define QSM_IARB            0x7  

#define TPU_IARB            0xC

#define PIT_IARB            0x5







typedef struct

{

	float fparm1;

	float fparm2;

	long  lparm3;

	long  lparm4;

} TMBUFF; 









#endif

