
#include "stddef.h"
#include "tpu_reg.h"
#include "tpu.h"
#include "fqd.h"

// Read the position of a quadrature encoder	//  and reset to zero.
// usage:
// x = quad_read(chan)

short int quad_read(int chan)
{
short int quad_count;
quad_count = *(short int *)(TPU_RAM + chan*16 + 2);
(*(short int *)(TPU_RAM + chan*16 + 2)) = 0; /* Reset channel counter after reading it. */
  return (quad_count);
}

// Initialize quadrature encoder. Pass channel A, channel B is assumed
// to be the next TPU channel
// usage:
// quad_init(chan)

int quad_init(int chan)
 {
  tpu_set_cpr(chan+0, 0);	/* Disable the Channel Priority Register */
  tpu_set_cpr(chan+1, 0);
  tpu_set_cfsr(chan+0, 6);	/* Set FQD Funciton code */
  tpu_set_cfsr(chan+1, 6);
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0x8)) = (chan+1)*16+6;
  (*(short int *)(TPU_RAM + (chan+1)*16 + 0x8)) = (chan+0)*16+6;
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0xa)) = chan*16+1;
  (*(short int *)(TPU_RAM + (chan+1)*16 + 0xa)) = chan*16+1;
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0x2)) = 0;
  tpu_set_hsqr(chan+0, 0);	/* Set Host Sequence Register */
  tpu_set_hsqr(chan+1, 1);
  tpu_set_hsrr(chan+0, 3);	/* Set Host Service Request Register */
  tpu_set_hsrr(chan+1, 3);
  tpu_set_cpr(chan+0, 3);	/* Set Channel Priority Register */
  tpu_set_cpr(chan+1, 3);
  return 0;
}




int fqd_init(int chan) {

  tpu_set_cpr(chan+0, 0);	/* Disable the Channel Priority Register */
  tpu_set_cpr(chan+1, 0);
  tpu_set_cfsr(chan+0, 6);	/* Set FQD Funciton code */
  tpu_set_cfsr(chan+1, 6);
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0x8)) = (chan+1)*16+6;
  (*(short int *)(TPU_RAM + (chan+1)*16 + 0x8)) = (chan+0)*16+6;
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0xa)) = chan*16+1;
  (*(short int *)(TPU_RAM + (chan+1)*16 + 0xa)) = chan*16+1;
  (*(short int *)(TPU_RAM + (chan+0)*16 + 0x2)) = 0;
  tpu_set_hsqr(chan+0, 0);	/* Set Host Sequence Register */
  tpu_set_hsqr(chan+1, 1);
  tpu_set_hsrr(chan+0, 3);	/* Set Host Service Request Register */
  tpu_set_hsrr(chan+1, 3);
  tpu_set_cpr(chan+0, 3);	/* Set Channel Priority Register */
  tpu_set_cpr(chan+1, 3);

  return 0;
}

short int fqd_get_count(int chan) {

  return (*(short int *)(TPU_RAM + chan*16 + 2));
}


