#include "lcd.h"
#include "sim.h"
unsigned int string_length(unsigned char buffer[])
{
	unsigned int i = 0;
	while(buffer[i] != '\0')
	{
		i++;
	}
	return(i);
}

void reverse(char s[])
{
	int c,d,e;
	for(d = 0, e=string_length(s)-1;d<e;d++,e--)
	{
		c=s[d];
		s[d] = s[e];
		s[e] = c;
	}
	return;
}

unsigned char to_hex(unsigned char val)
{
	unsigned char holding;
	if(val <= 9)
	{
		holding = ('0' + val);
	}
	else
	{
		holding = ('A' + (val - 10));		
	}
	return(holding);	
}

void ctohex(unsigned char num, char s[])
{
	s[0] = '0';
	s[1] = 'x';
	s[4] = '\0';
	s[3] = to_hex(num & 0x0F);
	s[2] = to_hex((num & 0xF0) >> 4);
	return;
}

void ctohex2(unsigned char num, char s[])
{
	s[2] = '\0';
	s[1] = to_hex(num & 0x0F);
	s[0] = to_hex((num & 0xF0) >> 4);
	return;
}

void itoa(int n, char s[])
{
	int i, sign;
	if((sign = n) < 0)
		n= -n;
	i = 0;
	do
	{
		s[i++] = n% 10 + '0';
	} while ((n/= 10) > 0);
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	reverse(s);
	return;
}

void delay(long l)
{
	long d;
	for(d=0;d<l;d++);
}

void lcd_wait(void)
{
	delay(4 * delay_1us);	// added more delay... also in header file.
}

// Waits for the busy flag on the LCD
// Usage:
//	lcd_wait()
void lcd_wait_new(void)
{
	while (LCD_MEMORY_LOCATION > 0x7f); 
	return;
}

void lcd_init(void)
{
	char *k;
	unsigned short int *w;
	w = (unsigned short int*)CSPAR0;	/* set up chip select pin asignment register (CSPAR)*/
	*w &= ~0x0080;
	*w |= 0x0080;
	w = (unsigned short int*)CSBAR2;	/* set chip select base address register (CSBAR)*/
	*w = (unsigned short int)((LCD_MEMORY_LOCATION >> 8) | BLOCK_SIZE_1MEG);
	w = (unsigned short int*)CSOR2;		/* set chip select base address register (CSBAR)*/
	*w = 0x7bf0;						/* this one uses external bus cycle termination */

	w = (unsigned short int*)PEPAR;		/* PEPAR, set the bit to enable the DSACK0 strobe rather than the port E pin*/
	*w |= 0x0001;
	k = (char*)LCD_MEMORY_LOCATION;  	/* now point at the page where /cs2 is hooked up */
	
	//	------------
	//	init steps from lcd spec sheet
	*k = 0x3c;
	delay(10 * delay_1ms);
	*k = 0x3c;				//	8-bit, 2 lines, 5x10
	delay(200 * delay_1us);
	*k = 0x3c;				//	8-bit, 2 lines, 5x10
	delay(200 * delay_1us);
	*k = 0x3c;				//	8-bit, 2 lines, 5x10
	delay(200 * delay_1us);
	*k = 0x3c;	//0x06			//	8-bit, 2 lines, 5x10
	delay(200 * delay_1us);
	*k = 0x08;	//0x0F			//	disp OFF
	delay(10 * delay_1ms);
	*k = 0x0c;	//0x01			//	disp ON, cursor off, blink off
	delay(10 * delay_1ms);
	*k = 0x06;	//0x80			//	move curs
	delay(delay_1ms);
	//	------------return here
	
	*k = 0x80;				//	addr=0
	delay(delay_1ms);

	*k = 0x01;				//	clear
	delay(5 * delay_1ms);

	*k = 0x02;				//	home
	delay(5 * delay_1ms);

	return;
}

void lcd_clear(void)
{
	char *k;
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0x01;
	lcd_wait();
	*k = 0x02;
	lcd_wait();
	return;
}

void lcd_line1(void)
{
	char *k;
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0x80;
	lcd_wait();
	return;
}

void lcd_line2(void)
{
	char *k;
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0xC0;
	lcd_wait();
	return;
}
void lcd_line3(void)
{
	char *k;
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0x94;
	lcd_wait();
	return;
}
void lcd_line4(void)
{
	char *k;
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0xD4;
	lcd_wait();
	return;
}

/* ********** */

void scrollr(int e,int n)  // e is no of char to scroll n is how fast
{
int s;
char *k;
for(s = 0; s < e; s++)
	{
	delay(n * delay_1ms); 
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0x1C;	// 28
	lcd_wait();
	}	// next
return;
}	// return

void scrolll(int e, int n)  // e is no of char to scroll n is how fast
{
int s;
char *k;
for(s = 0; s < e; s++)
	{
	delay(n * delay_1ms);
	k = (char*)LCD_MEMORY_LOCATION;
	*k = 0x18;	// 24
	lcd_wait();
	}
return;
}

/* ********** */



void lcd_string(char* data)
{
	char *k;
	k = (char*)(LCD_MEMORY_LOCATION + 1);
	while(0 != *data)
	{
		*k = *data;
		lcd_wait();
		data++;
	}
	return;
}
