/* Main routines for Maxbot
*  Created 12-29-2001
*  Tom Light (All rights Reserved)
*/

/* ************************************** */
/* Include Files						  */
/* ************************************** */
#include <stdio.h>
#include "maxbot.h"
#include "sim.h"
#include "sim_reg.h"
#include "qsm_reg.h"

#include "pwm.c"					/* pwm and dio routines use QOM function */
#include "tpu.c"					/* Compiler will suck in the code here for now */
#include "lcd.c"
#include "fqd.c"					/* Quadrature encoder functions */
#include "mcpwm.c"					/* MCPWM motor drive functions */
#include "qsm.c"					/* SCI serial functions */
#include "sonar.c"					/* Sonar init, read and related functions */

#define WALLSPEED 50
/* ************************************** */
/* Global variables */
/* ************************************** */

MotorInfo Left, Right;
SonarInfo RfSonar, LfSonar;
volatile unsigned short drive = 0xff;
static short ir_r, ir_l, ir_c;
static short ir_rd, ir_ld, ir_cd;
static char do_task;

volatile long tmp_Ldist, tmp_Rdist = 0;

// int iX, iY, iTheta;
// float X, Y, Theta;
// volatile int count = 0;
// volatile int count1 = 0;


int main(void) {

//	int x=1,y=1;
	int j, i;
	int period = 0x1FF;
	int pwm = period/2;
	unsigned int sys_clock;
	unsigned int baudrate;
	char tmparr[80];
	tmp_Ldist = tmp_Rdist = 0;

	/* ************************************** */
	/* INTERRUPT VECTOR TABLE ASSIGNMENTS     */
	/* ************************************** */
	/* Initialize all interrrupt vectors to default_int() */

	for(j=2;j<256;j++) *(long*)(vbraddr+j*4) = *(long*)(j*4);

	/* Update the VBR for new vector table location */
	asm("
		move %d0,-(%a7);			/* save regs */
		move #0x400,%d0;			/* load new vector */
		movec %d0,%vbr;				/* store in vector base regs */
		move (%a7)+,%d0");			/* restore regs */


	/* Set up first user interrupt vector (64) to point to TPU vectors */
	/* TPU uses next 15 vectors so next available (80) is set to hz100_int*/

	*(long*)(vbraddr+64 * 4) = (long)tpu_int;		/* 1st TPU interrupt handler */
	*(long*)(vbraddr+80 * 4) = (long)hz100_int;		/* PIT interrupt handler */
	*(long*)(vbraddr+SCIVEC * 4) = (long)sci_int;	/* SCI interrupt handler SCIVEC = 0x56 */

	/* ************************************** */
	/* END INTERRUPT VECTOR TABLE ASSIGNMENTS */
	/* ************************************** */

	/* Set System Clock Speed */
	// SIM_SYNCR = 0x7F00;			/* Set clock speed to 16.777Mhz */
	// SIM_SYNCR = 0xA600;			/* Set clock speed to 20.447Mhz */
	// SIM_SYNCR = 0xD700;			/* Set clock speed to 25.166Mhz */
	// SIM_SYNCR = 0xDE00;			/* Set clock speed to 32.506Mhz */
	// while((SIM_SYNCR & 0x8) != 0) ;  /* Wait for clock to lock on new speed. */

	/* Set 19200 Baud to match clock speed*/
	// QSM_SCCR0 = 27;			/* 19200 baud 16.777Mhz system clock */
	// QSM_SCCR0 = 21;			/* 19200 baud 20.447Mhz system clock */
	// QSM_SCCR0 = 29;			/* 19200 baud 25.166Mhz system clock */
	// QSM_SCCR0 = 35;			/* 19200 baud 32.506Mhz system clock */

/* ************************************** */

/* Initialize TPU Supervisor mode. System clock/4 default is /32 */

    tpu_init(0xE);				 	/* Iarb ix 0xE (second highest arbitration level). */
    tpu_set_ticr(0x6, 0x4);			/* 0x6, 0x5 if 0x4 chan0 irq = 0x40 chan1 irq =0x41 etc. */

	lcd_init();

/*	irq_rate = irq_rate * period * TPUticks. @16Mhz, 1 TPU tic = 250nsec
*	TCR1tics * TCR1rate  ns/period    ns / sec
*	so 511tics * 250nsec = 127750ns and 1,000,000,000 / 127750 = 7827.79 periods per sec
*	127750ns = .127750 ms so for a 20ms rate irq_rate needs to be 157 and for 10ms it's 78
*/

	mcpwm_init(0, 3, period, 157);	/* master channel (chan, pri, period, irq_rate15) */
	mcpwm_add_ea(1, pwm);			/* Left motor - slave channel 1 edge aligned */
	mcpwm_add_ea(2, pwm);			/* Right motor - slave channel 2 edge aligned */
	mcpwm_start();
	quad_init(6);					/* Left quadrature encoder init */
	quad_init(8);					/* Right quadrature encoder init */
	dio_init(4,0);					/* Left  motor enable (active low) */
	dio_init(5,0);					/* Right motor enable (active low) */
	SCIinit();
	MotorInit();
//	sonar_init();
	ir_init();

/* ************************************** */
	init_port_pin(PORTF, 4, 0, 0);			// Menu INPUT - PORT Pin, dir, state
	init_port_pin(PORTF, 5, 0, 0);			// Menu INPUT - PORT Pin, dir, state
	init_port_pin(PORTF, 6, 0, 0);			// Menu SELECT - 0 is input 1 is output
/* ************************************** */


	lcd_line1();
	lcd_string("MAXBOT Clock:");
	sys_clock = 0;
	sys_clock = SIM_SYNCR;
	sys_clock = sys_clock >> 8;		// look at high byte for system speed
/* Eventually use this to set the SCI baud rate */
	if (sys_clock == 127 || sys_clock == 159 || sys_clock == 207) lcd_string(" 16Mhz");
	if (sys_clock == 166) lcd_string(" 20Mhz");
	if (sys_clock == 215 || sys_clock == 175) lcd_string(" 25Mhz");
	if (sys_clock == 222 || sys_clock == 189) lcd_string(" 32Mhz");
	for(i=0;i<50000L;i++) ;

/* Period = (PITR * 4) / 32768 */
//	SIM_PITR = 0x0110;					/* Set PITR for 1Hz period */
	SIM_PITR = 0x0052;					/* Set PITR for 100Hz period(99.90Hz) */
	SIM_PICR = 0x0650;					/* Level 6 int, vec num 0x50 (80 dec) */
	asm("move.w #0x2500,%sr");			/* Enable level 6&7 interrupts */
 
/* ************************************** */
/* Background tasks begin here */
/* ************************************** */
// do_task = FINDWALL;
// do_task = WALLLEFT;
// do_task = WALLRIGHT;
do_task = NONE;

	while(1)
	{
//	sonar_read();		// result stored in rf_sonar, lf_sonar, r_sonar, l_sonar
//	sonar_update();		// updates the sonar structure vars

	menu();

	switch(do_task)
	{
		case NONE:
			break;
		case FINDWALL:
			findwall();
			break;
		case WALLLEFT:
			wallfollow_l();
			break;
		case WALLRIGHT:
			wallfollow_r();
			break;
		default:
		break;
	}


//	lcd_line1();
//	lcd_string("                    ");
//	lcd_line1();
//	itoa(RfSonar.dist[0], char_buffer);
//	lcd_string("RF:");
//	lcd_string(char_buffer);

//	lcd_line2();
//	lcd_string("                    ");
//	lcd_line2();
//	itoa(LfSonar.dist[0], char_buffer);
//	lcd_string("LF:");
//	lcd_string(char_buffer);

	lcd_line2();
	lcd_string("                    ");
	lcd_line2();
	itoa(ir_rd, char_buffer);
//	itoa(RfSonar.dist[0], char_buffer);
	lcd_string("R: ");
	lcd_string(char_buffer);

//	lcd_line3();
//	lcd_string("                    ");
//	lcd_line3();
	itoa(ir_cd, char_buffer);
//	itoa(LfSonar.dist[0], char_buffer);
	lcd_string(" C: ");
	lcd_string(char_buffer);

//	lcd_line4();
//	lcd_string("                    ");
//	lcd_line4();
	itoa(ir_ld, char_buffer);
	lcd_string(" L: ");
	lcd_string(char_buffer);

	for(i=0;i<10000L;i++);

	}	/* End while(1) */

								/* Should never get here */
	asm("move.w #0x2700,%sr");	/* Disable all interrupts (status register) word mode */
	exit(0);					/* Return to debug */
}	/* End of Main*/

/* ************************************** */
/* Put sensor, motor, etc routines here */
/* ************************************** */

void ir_init(void)
{
	// Right GP2D02 IR
	init_port_pin(PORTE, 2, 1, 1);			// Green OUTPUT PORT Pin, dir, state
	init_port_pin(PORTE, 3, 0, 0);			// Yellow INPUT  0 is input 1 is output

	// Left GP2D02 IR
	init_port_pin(PORTE, 4, 1, 1);			// Green OUTPUT PORT Pin, dir, state
	init_port_pin(PORTE, 5, 0, 0);			// Yellow INPUT  0 is input 1 is output

	// Center GP2D02 IR
	init_port_pin(PORTE, 6, 1, 1);			// Green OUTPUT PORT Pin, dir, state
	init_port_pin(PORTE, 7, 0, 0);			// Yellow INPUT  0 is input 1 is output
	return;
}

/* ************************************** */

/* GP2D02 IR routine
 Generates required signals to get data out of IR range sensor.
 Valid Port is PORTE or PORTF
 Valid Pin is between 0 and 6
 NOTE: Pass output pin to Vin on the IR (green wire),
 input from IR is assumed to be the next pin on the port (yellow wire).
 Usage:
	x = Read_Ir(PORTx, OutPin)
*/

int read_ir( unsigned char *port, int outpin)
{
	unsigned char vout_mask, vin_mask;
	unsigned char w,x,z;
	unsigned short int range, y;
	vout_mask = 1 << (outpin+1);				// Move the bit masks into position
	vin_mask  = 1 << outpin;					// In mask is output from 68332 Port pin
	y = 0;
	for (x=1; x<4; x++)							// Average four readings
	{
		for (z=0; z<30; z++);					// Allow sensor 1.5ms time to recover from last distance reading.
		*port = *port & ~vin_mask;				// Pull Vin to ir low to start reading
		while (!(*port & vout_mask))			// vin_mask Loop until Vout goes high
		{										// Data sheet says wait at least 70ms
		}
		range = 0;								// Clear variable
		for (z=0; z<7; z++)						// Shift in eight bits...
		{
			range = range << 1;					// Shift bits left
			*port = *port | vin_mask;			// Pull Vin high
			for (w=0; w<2; w++);				// Wait a bit (.1ms) for the right pulse width
			*port = *port & ~vin_mask;			// Pull Vin low
			for (w=0; w<2; w++);				// Wait a bit (.1ms) for the right pulse width
			if (*port & vout_mask)				// Read "Vout" in thru E3
			{
				range++;						// Load "1" into LSB if set
			}
		}
		*port = *port | vin_mask;				// Pull Vin high
		y = y + range;
		for (z=0; z<30; z++);					// Allow sensor 1.5ms time to recover from last distance reading.
	}
	range = y >> 2;
	return range;								// Return accumulated data divided by 4
}

/* ************************************** */

void MotorInit(void)
{
	Left.Setpoint = Right.Setpoint = 0;			// 512;
	Left.Encoder  = Right.Encoder  = 0;
	Left.Distance = Right.Distance = 0;

	Left.Kp = Right.Kp = 30;		// 30
	Left.Kd = Right.Kd = 10;		// 10
	Left.Ki = Right.Ki = 10;		// 3
	Left.Ko = Right.Ko = 10;		// 10 Output Scale

	Left.Velocity = Right.Velocity = 0;
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	//was 35. Turned off for wallfollow for now.
	Left.Acceleration = Right.Acceleration = 0x0070;	// 0x0007
	Left.Maxoutput = Right.Maxoutput = 0xFF;
	Left.Clipoutput = Right.Clipoutput = 1;
}

/* ************************************** */

/* Wheel Base is 7" Perimeter is 21.99"
/* Wheel Diameter is 3.280" Perimeter is 10.3" */
/* Encoder Counter overflows at 329.74" */
/* Distance will overflow at 16959.96 feet (over 3 mi) */

/* ************************************** */

/* Not used right now
* void doMotion(MotorInfo *p)
* Called at the loop rate to add "velocity" to the setpoint thus effecting a motion.
* Velocity is ramped up and down by "acceleration"
* Velocity and Acceleration are 8.8 numbers.  Velocity is added to the setpoint (an 24.8 number)
*/

void doMotion(MotorInfo *p)
{
	if (p->Velocity < p->VelocitySetpoint)
	{
		p->Velocity += p->Acceleration;
		if (p->Velocity > p->VelocitySetpoint)
			p->Velocity = p->VelocitySetpoint;
	}
	else 
	{
		p->Velocity -= p->Acceleration;
		if (p->Velocity < p->VelocitySetpoint)
		p->Velocity = p->VelocitySetpoint;
	}
		p->Setpoint += p->Velocity;
}

/* ************************************** */

int pid(MotorInfo *p, int chan)
{
	int Perror, output;

	p->Encoder = quad_read(chan);		/* quad_read resets counter each read so */
	p->Distance += p->Encoder;			/* Keep an accumulated count */

	Perror = p->VelocitySetpoint - p->Encoder; 

	output = (p->Kp * Perror + p->Kd * (Perror - p->PrevErr) + p->Ki * p->Ierror)/p->Ko; 
	p->PrevErr = Perror;

    if (output >= (p->Maxoutput - p->Clipoutput))
		{
		output = (p->Maxoutput - p->Clipoutput);
		}	
    else if (output <= (-(p->Maxoutput) + p->Clipoutput))
		{
		output = (-(p->Maxoutput) + p->Clipoutput);
		}
    else
		{
		p->Ierror += Perror;
		}
	p->LastEncoder = p->Encoder;
	return (p->Maxoutput + output);		/* Convert the output to LAP */
}

/* ************************************** */

void findwall(void)
{
	int x;
	short nearest_r, nearest_l;
	ir_r = read_ir( PORTE, 2);			// Right ir
	ir_l = read_ir( PORTE, 4);			// Left ir
	ir_c = read_ir( PORTE, 6);			// Center ir

	if (ir_r < 12 && ir_c < 12 && ir_l < 19)	//  Out of ir range 12 = (l-41" c-26" r-30")
	{
		Left.VelocitySetpoint = Right.VelocitySetpoint = 70;
		while (ir_c < 15)				// move fast to wall then slow down
		{
			ir_c = read_ir( PORTE, 6);	// Center ir
			for(x=0;x<5000;x++);
		}
		Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;
		while (ir_c < 25)				// get a bit closer then turn on wallfollow
		{
			ir_c = read_ir( PORTE, 6);		// Center ir
			for(x=0;x<5000;x++);
		}
		ir_r = read_ir( PORTE, 2);		// Right ir
		ir_l = read_ir( PORTE, 4);		// Left ir
		if ( (ir_l -5) > ir_r)			// Do closest routine
		{
			do_task = WALLLEFT;
			return;
		}
		else
		{
			do_task = WALLRIGHT;
			return;
		}
	}
	else
	{
		do_task = WALLRIGHT;
		return;
	}
return;
}

/* ************************************** */

void wallfollow_r(void)
{
	ir_r = read_ir( PORTE, 2);			// Right ir
//	ir_l = read_ir( PORTE, 4);			// Left ir
	ir_c = read_ir( PORTE, 6);			// Center ir

	ir_rd = ir_r;
	ir_ld = ir_l;
	ir_cd = ir_c;
// TEST CODE (untested)
//	if(ir_r <10)	// Sensor is out of range, go find another wall to follow.
//		{
//		do_task = FINDWALL;
//		return;
//		}
// END TEST CODE
	ir_r = (30 - ir_r) >> 1;	// approx 17"
//	ir_l = (33 - ir_l) >> 1;	// approx 17"
	if(ir_c < 40)		// 8" approx Don't run into anything
		{
		Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;	// Return to normal fwd
	 	Left.VelocitySetpoint += ir_r;		// // // // 5;
		Right.VelocitySetpoint -= ir_r;		// 5;
		}
	else	// Front is blocked
		{
		while (ir_c > 20)		// 16" approx
			{
			l_turn_45();
			ir_c = read_ir( PORTE, 6);			// Center ir
			}
 		Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;	// Return to normal fwd
		}	// end else
return;
}

/* ************************************** */

void wallfollow_l(void)
{
//	ir_r = read_ir( PORTE, 2);			// Right ir
	ir_l = read_ir( PORTE, 4);			// Left ir
	ir_c = read_ir( PORTE, 6);			// Center ir

	ir_rd = ir_r;
	ir_ld = ir_l;
	ir_cd = ir_c;
// TEST CODE (untested)
//	if(ir_l <10)	// Sensor is out of range, go find another wall to follow.
//		{
//		do_task = FINDWALL;
//		return;
//		}
// END TEST CODE
//	ir_r = (30 - ir_r) >> 1;	// approx 17"
	ir_l = (33 - ir_l) >> 1;	// approx 17"
	if(ir_c < 40)		// Don't run into anything
		{
		Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;	// Return to normal fwd
 		Left.VelocitySetpoint -= ir_l;		// 5;
		Right.VelocitySetpoint += ir_l;		// // // // 5;
		}
	else	// Front is blocked
		{
		while (ir_c > 20)		// 16" approx
			{
			r_turn_45();
			ir_c = read_ir( PORTE, 6);			// Center ir
			}
 		Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;	// Return to normal fwd
		}	// end else
return;
}

/* ************************************** */

void r_turn_45(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop motors

	tmp_Ldist = (Left.Distance + 546);	// Tics to Turn 45 degrees
	tmp_Rdist = (Right.Distance - 546);
	Left.VelocitySetpoint = 25;
	Right.VelocitySetpoint = -25;
	while ( tmp_Ldist > Left.Distance || tmp_Rdist < Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}

/* ************************************** */

void l_turn_45(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop motors

	tmp_Ldist = (Left.Distance - 546);	// Tics to Turn 45 degrees
	tmp_Rdist = (Right.Distance + 546);
	Left.VelocitySetpoint = -25;
	Right.VelocitySetpoint = +25;

	while ( tmp_Ldist < Left.Distance || tmp_Rdist > Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}

/* ************************************** */

void r_turn_90(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop, then start a slow turn

	tmp_Ldist = (Left.Distance + 1093);	// Tics to Turn 180 degrees
	tmp_Rdist = (Right.Distance - 1093);
	Left.VelocitySetpoint = 25;
	Right.VelocitySetpoint = -25;
	while ( tmp_Ldist > Left.Distance || tmp_Rdist < Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}

/* ************************************** */

void l_turn_90(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop, then start a slow turn

	tmp_Ldist = (Left.Distance - 1093);	// Tics to Turn 180 degrees
	tmp_Rdist = (Right.Distance + 1093);
	Left.VelocitySetpoint = -25;
	Right.VelocitySetpoint = +25;

	while ( tmp_Ldist < Left.Distance || tmp_Rdist > Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}
/* ************************************** */

void r_turn_180(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop, then start a slow turn

	tmp_Ldist = (Left.Distance + 2186);	// Tics to Turn 180 degrees
	tmp_Rdist = (Right.Distance - 2186);
	Left.VelocitySetpoint = 25;
	Right.VelocitySetpoint = -25;
	while ( tmp_Ldist > Left.Distance || tmp_Rdist < Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}

/* ************************************** */

void l_turn_180(void)
{
	Left.VelocitySetpoint = Right.VelocitySetpoint = 0;	// Stop, then start a slow turn

	tmp_Ldist = (Left.Distance - 2186);	// Tics to Turn 180 degrees
	tmp_Rdist = (Right.Distance + 2186);
	Left.VelocitySetpoint = -25;
	Right.VelocitySetpoint = +25;

	while ( tmp_Ldist < Left.Distance || tmp_Rdist > Right.Distance )
	{
		if (Left.Distance == tmp_Ldist)
		{
			Left.VelocitySetpoint = 0;
		}
		if (Right.Distance == tmp_Rdist)
		{
			Right.VelocitySetpoint = 0;
		}
	}
}

/* ************************************** */

void menu(void)
{
int y;
static char x = 0;
char *menu[] = {

"None                ",
"Find Wall           ",
"Wall Left           ",
"Wall Right          "};

	if ( !( *PORTF & 0x10 ))	// UP Menu switch pressed ? (active low)
	{
// SCIPutS("Dn Menu Button Pressed \n \r");
		x++;
		if(x > 3) x=0; 
		lcd_line4();
		lcd_string( menu[x] );
		for(y=0;y<10000;y++);
	}

	if ( !( *PORTF & 0x20 ))	// Down Menu switch pressed ? (active low)
	{
// SCIPutS("Up Menu Button Pressed \n \r");
		x--;
		if(x < 0) x=3; 
		lcd_line4();
		lcd_string( menu[x] );
		for(y=0;y<10000;y++);
	}


	if ( !( *PORTF & 0x40 ))	// Run Menu switch pressed ? (active low)
	{
		do_task = x;	
// SCIPutS("Run Button Pressed \n \r");
		for(y=0;y<10000;y++);
	}
}


/* ************************************** */
/* Interrupt service routines begin here */
/* ************************************** */

void hz100_handler()		/* Called 100 times per sec by the Periodic Interrupt Timer */
{

// Need a 20hz motor loop
// 20hz encoder loop
// 10hz ir loop
// 10hz sonar loop
	// 10ms  = 100hz
	// 25ms  = 40hz
	// 50ms  = 20hz
	// 100ms = 10hz
	// 200ms = 5hz
// motor_loop++;
// ir_loop++;
// sonar_loop++;

}
void tpu_handler(void)
{
	volatile int tmpi;

	tmpi = TPU_CISR;
	TPU_CISR = tmpi & 0xFFFE;	// Clear interrupt bit 0 (0 1 2 ...) MCPWM master

	drive = pid(&Left, 6);
	mcpwm_set_ea_hightime(1,drive);	// left motor

	drive = pid(&Right, 8);
	mcpwm_set_ea_hightime(2,drive);	// Right motor

}

/* ************************************** */
/* Interrupt Handler vectors */
/* ************************************** */

// default interrupt
//asm(".global default_int;default_int:link %a6,#0;loop:bgnd;jmp loop;unlk %a6;rte");
//  asm(".global __vector_default;__vector_default:loop:bgnd;jmp loop;rte");
asm(".global illegal_instruction_int;illegal_instruction_int:ill_loop:jmp ill_loop;rte");

/* 100Hz clock PIT interrupt */
asm("
	.global hz100_int;
	hz100_int:
	link %a6,#0;
	moveml %a6-%d0,-(%a7);
	jsr hz100_handler;
	moveml (%a7)+,%d0-%a6;
	unlk %a6;
	rte;");

/* TPU interrupt */
asm("
	.global tpu_int;
	tpu_int:
	link %a6, #0;
	moveml %a6-%d0,-(%a7);
	jsr tpu_handler;
	moveml (%a7)+,%d0-%a6;
	unlk %a6;
	rte;");

/* SCI interrupt */
asm("
    .global sci_int;
    sci_int:
	link %a6, #0;
	moveml %d0-%d7/%a0-%a6, %SP@-;
	jsr sciint;
	moveml %SP@+, %d0-%d7/%a0-%a6;
	unlk %a6;
	rte;");


/*
 The TPU1 has a 250 n_sec period @16.78Mhz
 The TPU1 has a 123 n_sec period @32.506Mhz
 1 second = 1,000,000,000 nanosecond (n_sec)
 1 second = 1,000,000 microsecond (u_sec)
 1 second = 1,000 millisecond (m_sec)
*/
/* MUCH better code if you use the bitwise operations rather than bitfield. e.g.
*
*	#define bit(x)  (1 << (x))
*
*	a |= bit(1);    // sets a bit
*	a &= ~bit(1);   // clears a bit
*	a ^= bit(1);    // toggle
*
*	if (a & bit(1)  // test
*/
/* ************************************** */
void findwall_old(void)
{
	char x;
	short nearest_r, nearest_l;
	ir_r = read_ir( PORTE, 2);			// Right ir
	ir_l = read_ir( PORTE, 4);			// Left ir
	ir_c = read_ir( PORTE, 6);			// Center ir

// TEST CODE (untested)
	if (ir_r < 10 && ir_l < 10 && ir_c < 10)	// Out of ir range 12 = (l-41" c-26" r-30")
	{
		sonar_read();		// result stored in rf_sonar, lf_sonar structures
		sonar_update();		// updates the sonar structure vars (stores last 8 readings)
		if (RfSonar.dist[0] < 3600 || LfSonar.dist[0] < 3600)
		{
			Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;
			if(RfSonar.dist[0] > LfSonar.dist[0])
			{
				do_task = WALLLEFT;
				return;
			}
			else
			{
				do_task = WALLRIGHT;
				return;
			}
		}
		else
		{
			for (x=0; x<8; x++)		// get 8 new readings.
			{
				sonar_read();
				sonar_update();
				l_turn_45();
				Left.VelocitySetpoint = Right.VelocitySetpoint = 0;

				if(RfSonar.dist[x-1] < RfSonar.dist[x])	// get shortest RfSonar
				{ nearest_r = x-1; } 
				else { nearest_r = x; }
			
				if(LfSonar.dist[x-1] < LfSonar.dist[x])	// get shortest RlSonar
				{ nearest_l = x-1; }
				else { nearest_l = x; }
			}
			if (nearest_r > nearest_l)
			{
//				nearest_l++;
				for (x=0; x<nearest_l; x++)		// turn toward shortest reading
				{
					l_turn_45();
				}

				Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;
// check center ir
				do_task = WALLLEFT;
				return;
			}
			else
			{
//				nearest_r++;
				for (x=0; x<nearest_r; x++)
				{
					r_turn_45();
				}
				Left.VelocitySetpoint = Right.VelocitySetpoint = WALLSPEED;
// check center ir
				do_task = WALLRIGHT;
				return;
			}
		}
	}
// END TEST CODE
return;
}


