/* TMAX is max pwm fwd pulse */
// #define TMAX 255

/* ************************************** */
typedef struct
{
    long    Setpoint;           // Desired Motor Encoder in encoder counts
    long    Encoder;            // Current motor position in encoder counts
	long	Distance;			// Accumulated encoder distance
	short	LastEncoder;		// Previous motor position in encoder counts
    signed char Kp;				// Proportional term
    signed char Kd;				// Derivative term
    signed char Ki;				// Integral term
	signed char	Ko;				// Output scale value
    short	PrevErr;
    short	Ierror;
    short	Velocity;			// Current velocity?
    short	VelocitySetpoint;	// Set velocity (in counts per loop)
    short	Acceleration;		// Set acceleration (in counts per loop)

	short Maxoutput;			// Max allowable output from the PID.
	char Clipoutput;			// Limit the output by some amount in both the + and -
} MotorInfo, * pMotorInfo;

/* ************************************** */
typedef struct
{
	unsigned short	dist[8];	// Sonar distance values
//	unsigned short	current;	// Current Sonar reading
//	unsigned short	last1;		// Last Sonar reading
//	unsigned short	last2;		// Sonar history
//	unsigned short	last3;		// Used for reading 360 deg in 45 deg steps
//	unsigned short	last4;
//	unsigned short	last5;
//	unsigned short	last6;
//	unsigned short	last7;

} SonarInfo, * pSonarInfo;

unsigned char char_buffer[100];		/* Buffer for the LCD routines */

extern MotorInfo Left, Right;
extern SonarInfo RfSonar, LfSonar, RSonar, LSonar;

// extern float X, Y, Theta;
// extern int iX, iY, iTheta;      // Integers in 1/10" increment

// volatile int hours=0, minutes=0, seconds=0;
// volatile int quadl1=0, quadl2=0;
// volatile int quadr1=0, quadr2=0;
// volatile int motor_loop=0, ir_loop=0, sonar_loop=0;

// #define MAXOUTPUT 0xFF		// 0x7F

/* ************************************** */
/* Function Prototypes */
/* ************************************** */
extern void __vector_default(void);	/* Default interrupt handler */
void illegal_instruction_int(void); /* Default illegal instruction handler */
void hz100_int(void);				/* PIT timer interrupt */
void tpu_int(void);					/* TPU0 int pointer */
void sci_int(void);					/* SCI interrupt */
void MotorInit(void);
void doMotion(MotorInfo *p);
int pid(MotorInfo *p, int chan);
void ir_init(void);
int read_ir( unsigned char *port, int outpin);
void findwall(void);
void wallfollow_r(void);
void wallfollow_l(void);
void r_turn_45(void);
void l_turn_45(void);
void r_turn_90(void);
void l_turn_90(void);
void r_turn_180(void);
void l_turn_180(void);
void menu(void);
/* ************************************** */
/* System Defines */
/* ************************************** */
#define vbraddr 0x400				/* Location of the base vector address */
#define PORTE (unsigned char *) (SIM_BASE+0x11)
#define PORTF (unsigned char *) (SIM_BASE+0x19)
typedef unsigned short int word;

#define NONE		0
#define FINDWALL	1
#define	WALLLEFT	2
#define WALLRIGHT	3
 
/* ************************************** */
/* Device Pin Assignments */
/* ************************************** */

/*	Master MCPWM channel TPU0

*	Left motor pwm		 TPU1

*	Right motor pwm		 TPU2

*						 TPU3

*	Left motor enable	 TPU4

*	Right motor enable	 TPU5

*	Left quad encoder	 TPU6-7

*	Right quad encoder	 TPU8-9

*						 TPU10

*						 TPU11

*	Right Front Sonar:	PortF0 - Init	TPU 12 - Echo

*	Left Front Sonar:	PortF1 - Init	TPU 13 - Echo

*	Right Sonar:		PortF2 - Init	TPU 14 - Echo

*	Left Sonar:			PortF3 - Init	TPU 15 - Echo

*	Menu switch:		PortF4		Down

*	Menu switch:		PortF5		Up

*	Menu switch:		PortF6		Select

*/
