// #include "tpu_reg.h"
// #include "tpu.c"

static short prio;
static short master;
static short period;
static short slaves;
static short slavetype;


int mcpwm_init(int chan, int pri, int pd, int irqr)		// channel, priority, period, irq_rate
{
	unsigned short tmp;
	prio = pri;
	if (pd > 0x4000)
	{
		return -1;
	}
	master = chan;
	period = pd;
	tmp = (0xFF00 & irqr<<8) | 0x00FF;
	tpu_set_cfsr(chan, 0x7);
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x0)) = period;		/* Period */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x2)) = tmp;			/* H-byte irq rate, L-byte period count */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x8)) = chan * 16 + 4;	/* rtime ptr */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0xA)) = chan * 16 + 6;	/* ftime ptr */
	if (irqr != -1)
	{
		tpu_set_cier(chan, 1);	// Enable interrupt
	}
	else
	{
		tpu_set_cier(chan, 0);	// Disable interrupt
	}
	tpu_set_hsrr(chan, 0x3);
	slaves = 0;
	slavetype = 0;
	return 0;
}


int mcpwm_add_ea(int chan, int high)		// channel, hightime
{
	slaves |= 1<<chan;
	slavetype &= (0xFFFF^(1<<chan));
	tpu_set_cfsr(chan, 0x7);
	if (high > period)
	{
	high = period;
	}
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x0)) = period;			/* Period */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x2)) = high;				/* High time */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x6)) = chan * 16 + 2;		/* htime ptr */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0x8)) = master * 16 + 4;	/* rtime ptr */
	(*(unsigned short *)(TPU_RAM + chan*16 + 0xA)) = master * 16 + 6;	/* ftime ptr */
	tpu_set_cier(chan, 0);
	tpu_set_hsqr(chan, 0);
	tpu_set_hsrr(chan, 0x2);
return 0;
}


int mcpwm_set_ea_hightime(int chan, int high)		// channel, hightime
{
    (*(unsigned short *)(TPU_RAM + chan*16 + 0x2)) = high;

    return 0;
}


int mcpwm_start(void)
{
	int channel;
	tpu_set_cpr(master, prio);
	for (channel = 0; channel < 16; channel++)
	{
		/* Find the channel that needs to be configured as a slave */
		if ((slaves>>channel) & 0x0001)
		{
			tpu_set_cpr(channel, prio);
		}
	}
	return 0;
}
