/* 	qsm.h 

*/





#ifndef __QSM_h

#define	__QSM_h



/* Defines 

*/ 

#define qsize 160						/* Queue size - bytes */



/* Typedef for tx and rx queue structures for sci buffer routines

*/ 

typedef struct { 

	u_short in;        				 	/* Queue input/write pointer */ 

	u_short full;      				 	/* Queue full flag */

	u_short out;       				 	/* Queue output/read pointer */ 

	u_char q[qsize];  				 	/* Queue array */

} queue_struct;





struct QSMCR_ {

	volatile u_short	STOP:1;

	volatile u_short	FRZ:2;

	u_short	:5;

	volatile u_short	SUPV:1;

	u_short	:3;

	volatile u_short	IARB:4;

};

                    

struct QILR_ {

	u_char	:2;

	volatile u_char		ILQSPI:3;

	volatile u_char		ILSCI:3;

};



struct SCCR0_ {

	u_short :3;

	volatile u_short	SCBR:13;

};



struct SCCR1_ {

	u_short	:1;

	volatile u_short	LOOPS:1;

	volatile u_short	WOMS:1;

	volatile u_short	ILT:1;

	volatile u_short	PT:1;

	volatile u_short	PE:1;

	volatile u_short	M:1;

	volatile u_short	WAKE:1;

	volatile u_short	TIE:1;

	volatile u_short	TCIE:1;

	volatile u_short	RIE:1;

	volatile u_short	ILIE:1;

	volatile u_short	TE:1;

	volatile u_short	RE:1;

	volatile u_short	RWU:1;

	volatile u_short	SBK:1;

};



struct SPCR_ {

	/* now spcr0 */

	volatile u_short	MSTR:1;

	volatile u_short	WOMQ:1;

	volatile u_short	BITS:4;

	volatile u_short	CPOL:1;

	volatile u_short	CPHA:1;

	volatile u_short	SPBR:8;

	/* now spcr1 */

	volatile u_short	SPE:1;

	volatile u_short 	DSCKL:7;

	volatile u_short 	DTL:8;

	/* now spcr2 */

	volatile u_short	SPIFIE:1;

	volatile u_short	WREN:1;

	volatile u_short	WRT0:1;

	u_short	:1;

	volatile u_short	ENDQP:4;

	u_short	:4;

	volatile u_short	NEWQP:4;

	/* now spcr3 */

	u_short	:5;

	volatile u_short	LOOPQ:1;

	volatile u_short	HMIE:1;

	volatile u_short	HALT:1;

	volatile u_short	SPIF:1;

	volatile u_short	MODF:1;

	volatile u_short	HALTA:1;

	u_short	:1;

	volatile u_short	CPTQP:4;

};



/*

 * sr must not be defined as bitset, as it must be accessed in a single

 * read. Bitset operations are implemented by read-and-mask, so bits may

 * get masked out without being able to look at. At the next read the 

 * state might be gone.

 */

#define		SCSR_TDRE	(1<<8)		/* transm data reg empty */

#define		SCSR_TC		(1<<7)		/* transm complete flag */

#define		SCSR_RDRF	(1<<6)		/* rec data reg full */

#define		SCSR_RAF	(1<<5)		/* rec active */

#define		SCSR_IDLE	(1<<4)		/* line idle */

#define		SCSR_OR		(1<<3)		/* overrun */

#define		SCSR_NF		(1<<2)		/* noise */

#define		SCSR_FE		(1<<1)		/* framing error */

#define		SCSR_PF		(1<<0)		/* parity failed */



struct CR_ {

	volatile u_char	CONT:1;

	volatile u_char	BITSE:1;

	volatile u_char	DT:1;

	volatile u_char	DSCK:1;

	volatile u_char	PCS:4;

};



typedef struct qsm {

	struct 	QSMCR_ 				qsmcr;

	volatile u_short			qtest;

	struct	QILR_				qilr;

	volatile u_char				qivr;

	volatile u_short			qreserved1;

	struct	SCCR0_				sccr0;

	struct	SCCR1_				sccr1;

	volatile u_short			scsr;

	volatile u_short			scdr;

	volatile u_short			qreserved2;

	volatile u_short			qreserved3;

	volatile u_char				qreserved4;

	volatile u_char				portqs;

	volatile u_char				pqspar;

	volatile u_char				ddrqs;

	struct SPCR_ 				spcr;

	volatile u_short			qreserved5[0x70];

	volatile u_short			rr[0x10];

	volatile u_short			tr[0x10];

	struct CR_					cr[0x10];

} QSMREG;







/* Global Variables 

*/ 

queue_struct rxbuff, txbuff;			/* Transmit and receive queue structures */

unsigned char locbuf[10];



/* Function prototypes 

*/

void SCIPutS (char *message);

void SCIPutBytes(char *val, int len);

void qinit (queue_struct *sptr);		/* Initialize a specified buffer */

u_short qstat (queue_struct *sptr);		/* Return the status of a specified buffer */ 

void SCIinit ();                		/* SCI module initialization */

void QSPIinit ();                       /* QSPI module initialization */

void sciint ();             			/* SCI interrupt handler (TX and RX) */

char rx_byte (u_char *b);  		   		/* Read a byte from the RX buffer */

char tx_byte (u_char);            		/* Write a byte to the TX buffer */

void SCIPutInt(int num, int len);

void SCIPutHex(unsigned int num, int len);





#endif



