/*
	RM_RAM.LD
*/
/*
 * Linker script for typical ROM-based M68K embedded applications using COFF obj format.
 * Copyright  1997-1999 by Object Software Inc., All Rights Reserved.
 *
 * The copyright holder hereby grants permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 *
 */

 /*
	this script will place the entirety of the prgram into RAM
	on the Mini RoboMind

	set the variable __ram_size to be the size of the 
	RAM on your MRM minus __ram_start

	for example: my MRM has the 128KB upgrade. 128KB in bytes is
	128 * 1024 = 131072d = 20000h
	512 * 1024 = 524288d = 80000h

 */

STARTUP(rm_crt0.o)
OUTPUT_ARCH(m68k)
SEARCH_DIR(.)
GROUP(-lbcc -lc -lgcc -lm)
__DYNAMIC  =  0;


/*__ram_size   = 0x20000 - __ram_start;  128K ram */

__ram_start  = 0x03000;
__ram_size   = 0x7D000 - __ram_start;	/*0x7D000 512K ram 80000 */
__stack      = __ram_start + __ram_size - 0x4;
__prog_start = __ram_start + 0x10;

/*
	rm_crt0.s needs this to be defined
	(if it's zero, no image copy from rom to
	ram will occur)
*/
__data_start_rom = 0;


PROVIDE (__stack = __ram_start + __ram_size);
PROVIDE (__boot_stack = __ram_start + __ram_size);

PROVIDE (crt0_flags = 0);
PROVIDE (_crt0_flags = 0);
PROVIDE (hardware_init_hook = 0);
PROVIDE (_hardware_init_hook = 0);
PROVIDE (software_init_hook = 0);
PROVIDE (_software_init_hook = 0);


SECTIONS
{
  .text __ram_start :
  {
    LONG (0xbeefbeef);
    LONG (__stack);
    LONG (__prog_start);
    LONG (0x0);
    *(.text)
    . = ALIGN(0x4);
     __CTOR_LIST__ = .;
    LONG((__CTOR_END__ - __CTOR_LIST__) / 4 - 2)
    *(.ctors)
    LONG(0)
    __CTOR_END__ = .;
    __DTOR_LIST__ = .;
    LONG((__DTOR_END__ - __DTOR_LIST__) / 4 - 2)
    *(.dtors)
     LONG(0)
    __DTOR_END__ = .;
    *(.rodata)
    *(.gcc_exc)
    *(.gcc_except_table)

    __INIT_SECTION__ = . ;
    LONG (0x4e560000)	/* linkw %fp,#0 */
    *(.init)
    SHORT (0x4e5e)	/* unlk %fp */
    SHORT (0x4e75)	/* rts */

    __FINI_SECTION__ = . ;
    LONG (0x4e560000)	/* linkw %fp,#0 */
    *(.fini)
    SHORT (0x4e5e)	/* unlk %fp */
    SHORT (0x4e75)	/* rts */

/* hardware initialization lists go here */
    . = ALIGN (0x4);
    crt0_initialization_list = .;
    _crt0_initialization_list = .;
    *(.crt0ini)
    . = ALIGN (0x2);
    LONG (0)		/* null pointer terminates list */
    _etext = .;
    *(.lit)
    . = ALIGN (0x4);
  }

  .data :
  {
    __data_start = .;
    *(.shdata)
    *(.data)
    _edata = .;
  }

  .bss BLOCK (0x4) :
  {
    __bss_start = . ;
    *(.shbss)
    *(.bss)
    *(COMMON)
    *(.eh_fram)
    *(.eh_frame)
    _end =  ALIGN (0x8);
    __end = _end;
  }

  .stab 0 (NOLOAD) :
  {
    *(.stab)
  }

  .stabstr 0 (NOLOAD) :
  {
    *(.stabstr)
  }
}

