/* Port I/O routines and
 Devantech SRF04 Sonar routine
 Written by Tom Light

 The following subroutines are public domain

 Uses polled method to check interrupt flag on TPU channel
 NOTE: This routine was tested at 16.777Mhz and may need adjustment for other clock rates
 Init is on Port pin
 Echo is on TPU pin
*/

#include "sonar.h"

void sonar_init(void)					/* This routine initializes all sonar channels */
{
	init_port_pin(PORTF, 0, 1, 0);		// Pass: Port, Pin, Direction, State
	init_pta(12);						// Set channel 12 up for PTA function

	init_port_pin(PORTF, 1, 1, 0);		// Pass: Port, Pin, Direction, State
	init_pta(13);						// Set channel 13 up for PTA function

//	init_port_pin(PORTF, 2, 1, 0);		// Pass: Port, Pin, Direction, State
//	init_pta(14);						// Set channel 14 up for PTA function

//	init_port_pin(PORTF, 3, 1, 0);		// Pass: Port, Pin, Direction, State
//	init_pta(15);						// Set channel 15 up for PTA function

	return;
}
void sonar_read(void)					/* This routine reads all sonar channels vars are global initialized elsewhere*/
{
	int x;

	RfSonar.dist[0] = ping(PORTF, 0, 12);		// Pass: Port, Pin, TPU Channel
	for (x=0;x<400;x++);

	LfSonar.dist[0] = ping(PORTF, 1, 13);
//	for (x=0;x<400;x++);

//	RSonar.dist[0]  = ping(PORTF, 2, 14);
//	for (x=0;x<400;x++);

//	LSonar.dist[0]  = ping(PORTF, 3, 15);

return;
}

void sonar_update()
{
RfSonar.dist[7] = RfSonar.dist[6];
LfSonar.dist[7] = LfSonar.dist[6];
// RSonar.dist[7] = RSonar.dist[6];
// LSonar.dist[7] = LSonar.dist[6];

RfSonar.dist[6] = RfSonar.dist[5];
LfSonar.dist[6] = LfSonar.dist[5];
// RSonar.dist[6] = RSonar.dist[5];
// LSonar.dist[6] = LSonar.dist[5];

RfSonar.dist[5] = RfSonar.dist[4];
LfSonar.dist[5] = LfSonar.dist[4];
// RSonar.dist[5] = RSonar.dist[4];
// LSonar.dist[5] = LSonar.dist[4];

RfSonar.dist[4] = RfSonar.dist[3];
LfSonar.dist[4] = LfSonar.dist[3];
// RSonar.dist[4] = RSonar.dist[3];
// LSonar.dist[4] = LSonar.dist[3];

RfSonar.dist[3] = RfSonar.dist[2];
LfSonar.dist[3] = LfSonar.dist[2];
// RSonar.dist[3] = RSonar.dist[2];
// LSonar.dist[3] = LSonar.dist[2];

RfSonar.dist[2] = RfSonar.dist[1];
LfSonar.dist[2] = LfSonar.dist[1];
// RSonar.dist[2] = RSonar.dist[1];
// LSonar.dist[2] = LSonar.dist[1];

RfSonar.dist[1] = RfSonar.dist[0];
LfSonar.dist[1] = LfSonar.dist[0];
// RSonar.dist[1] = RSonar.dist[0];
// LSonar.dist[1] = LSonar.dist[0];
return;
}

/* Port initialization routine
 Initializes Port pin for general purpose I/O
 Valid Port is PORTE or PORTF defined as follows:
 #define PORTE (unsigned char *) (SIM_BASE+0x11)
 #define PORTF (unsigned char *) (SIM_BASE+0x19)
 Valid io_pin is between 0 and 7
 Valid io_dir is 0 Input or 1 Output
 Valid io_st is 0 initialized Low or 1 initialized High
 Usage:
	init_port_pin(Port, Pin, Direction, State)
*/

void init_port_pin(unsigned char *port, unsigned char pin, unsigned char dir, unsigned char st)
{
	unsigned char tmp = 1;
	st  = st  << pin;
	dir = dir << pin;
	tmp = tmp << pin;
	tmp = ~tmp;
	*(port + 6) = *(port + 6) & tmp;			// Pin Assignment Regs
	*(port + 4) = (*(port + 4) & tmp) | dir;	// Data Direction Regs
	*port = (*port & tmp) | st;
	return;
}

/* Set Port pin state
 Valid Port is PORTE or PORTF
 Valid Pin is between 0 and 7
 Valid State is 0 set Low or 1 set High
 Note: Pin0 (Modclk) is tied high thru a 10k res.
 Usage:
	set_pin(port, pin, st)
*/

void set_pin( unsigned char *port, unsigned char pin, unsigned char st)
{
unsigned char tmp = 1;
unsigned char x;
	st  = st  << pin;
	tmp = tmp << pin;
	tmp = ~tmp;
	*port = (*port & tmp) | st;
return;
}


/* Sonar PTA initialization routine
 For use with the Devantech SRF04 Ultrasonic Range Finder
 Initializes Tpu channel for PTA function for sonar ping
 Usage:
	init_pta(chan)
*/

int init_pta(unsigned char chan)						// Called by init_sonar above
{
int fl_time;
short int lwtime;

	tpu_set_cpr(chan, 0);								// channel, priority
	tpu_set_cfsr(chan, 0xF);							// F Set PTA funciton code
	tpu_set_cier(chan, 0);								// channel, logic Channel interrupt enable regs (disabled here)
	(*(short int *)(TPU_RAM + chan*16 + 0)) = 0x0F;		// Sets up channel control (high pulse using TCR1 timebase)
	(*(short int *)(TPU_RAM + chan*16 + 2)) = 0x0100;	// Max_count 1 (count 1 pulse)
	(*(short int *)(TPU_RAM + chan*16 + 6)) = 0;		// These must be cleared by cpu before hsr is set to %11
	(*(short int *)(TPU_RAM + chan*16 + 8)) = 0;
	(*(short int *)(TPU_RAM + chan*16 + 10)) = 0;
	tpu_set_hsqr(chan, 0);								// Host sequence register (00 high time accumulate)
	tpu_set_cpr(chan, 3);								// Channel Priority Register
//	tpu_set_cier(chan, 1);								// Channel, logic Channel interrupt enable regs (enabled here)
	tpu_set_hsrr(chan, 3);								// Host service request (%00 no service, %11 Initialize function)
 	fl_time = (*(short int *)(TPU_RAM + chan*16 + 8));
	lwtime = (*(short int *)(TPU_RAM + chan*16 + 10));

return 0;
}


/* Sonar Ping routine
 For use with the Devantech SRF04 Ultrasonic Range Finder
 Valid Port is PORTE or PORTF
 Valid Pin is between 0 and 7
 Valid TPU channel is between 0 and 15
 Uses TPU channel PTA function for sonar ping
 Usage:
	x = Ping(*port, pin, tpu_chan)
*/

int ping(unsigned char *port, unsigned char pin, unsigned char chan)
{
	unsigned int fl_time = 0;
	short int lwtime = 0;
	int x;
	set_pin(port, pin, 0);								// Make sure the init pin is off
	set_pin(port, pin, 1);								// Start the ping
//	for (x=0; x<270; x++);								// Delay a bit
	for (x=0; x<370; x++);								// Delay a bit
	set_pin(port, pin, 0);								// Turn the init pin off

// Can't wait for an interrupt that isn't turned on in the init routine!
//	while (tpu_get_cisr( chan ) == 0)					// See if flag is set
//	{
//	}
//	TPU_CISR = 0;										// Clear interrupt flag
	fl_time = (*(short int *)(TPU_RAM + chan*16 + 8)); 	// Get the high result
	lwtime = (*(short int *)(TPU_RAM + chan*16 + 10));	// Get the low result
	fl_time = fl_time << 16;							// Move it to the high word
	fl_time = fl_time + lwtime;							// Add in the low word
	fl_time = fl_time * 100;							// Show 2 place decimals (no decimal point)
	fl_time = fl_time / 295;							// Convert to inches (see note below)
	fl_time = fl_time >> 1;								// /2 Get one way flight time
	(*(short int *)(TPU_RAM + chan*16 + 8)) = 0;		// Clear the result registers
	(*(short int *)(TPU_RAM + chan*16 + 10)) = 0;
return fl_time;
}

/* ----------------------------------------
 Just some notes to keep me straight
 Conversions I'm using:
 Sound is
       1129 ft/sec
      13548 in/sec
    .013548 in/u_sec
 .000013548 in/n_sec

 The TPU1 has a 250 n_sec period @16mhz so:
 Sound is .003387 in/TPU period
 So it takes 295.24653 periods/inch
 So take the result periods / 295.24653 to get inches

 1 second = 1,000,000,000 nanosecond (n_sec)
 1 second = 1,000,000 microsecond (u_sec)
 1 second = 1,000 millisecond (m_sec)
 ---------------------------------------- */

